/**
 * @file
 * Grid implementation.
 *
 * @copyright Copyright © 2017–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                     "title" : "Make 3d Point Grid",
                     "keywords" : [ ],
                     "version" : "1.0.0",
                     "dependencies" : [ ],
                     "node": {
                          "exampleCompositions" : [ "CubeGrid.vuo" ]
                     }
                 });

void nodeEvent
(
        VuoInputData(VuoInteger, {"default": 10}) xAmount,
        VuoInputData(VuoInteger, {"default": 10}) yAmount,
        VuoInputData(VuoInteger, {"default": 10}) zAmount,
        VuoInputData(VuoPoint3d, {"default":{"x":-0.5, "y":0.5, "z":-1.0},"suggestedMin":{"x":-1.0, "y":-1.0, "z":-1.0}, "suggestedMax":{"x":1.0, "y":1.0, "z":1.0}}) rangeFrom,
        VuoInputData(VuoPoint3d, {"default":{"x":0.5, "y":-0.5, "z":0.0},"suggestedMin":{"x":-1.0, "y":-1.0, "z":-1.0}, "suggestedMax":{"x":1.0, "y":1.0, "z":1.0}}) rangeTo,
        VuoInputData(VuoCurve, {"default":"linear"}) curveX,
        VuoInputData(VuoCurve, {"default":"linear"}) curveY,
        VuoInputData(VuoCurve, {"default":"linear"}) curveZ,
        VuoInputData(VuoCurveEasing, {"default":"in"}) easingX,
        VuoInputData(VuoCurveEasing, {"default":"in"}) easingY,
        VuoInputData(VuoCurveEasing, {"default":"in"}) easingZ,
        VuoOutputData(VuoList_VuoPoint3d) list
)
{
    //Create the output list
    *list = VuoListCreate_VuoPoint3d();

    //Calculate points

    for(unsigned int k = 0; k < zAmount; ++k){
        for(unsigned int j = 0; j < yAmount; ++j){
            for(unsigned int i = 0; i < xAmount; ++i){

                VuoReal z = VuoReal_curve(k, rangeFrom.z, rangeTo.z, (zAmount - 1), curveZ, easingZ, VuoLoopType_None);
                VuoReal y = VuoReal_curve(j, rangeFrom.y, rangeTo.y, (yAmount - 1), curveY, easingY, VuoLoopType_None);
                VuoReal x = VuoReal_curve(i, rangeFrom.x, rangeTo.x, (xAmount - 1), curveX, easingX, VuoLoopType_None);

                VuoListAppendValue_VuoPoint3d(*list, VuoPoint3d_make(x, y, z));
            }
        }
    }
}
