/**
 * @file
 * Spherograph implementation.
 *
 * @copyright Copyright © 2012–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "math.h"

VuoModuleMetadata({
                     "title" : "Make Arc Points",
                     "keywords" : ["Magneson", "Circle segment", "circles", "lines" ],
                     "version" : "1.1.0",
                     "dependencies" : [ ],
                     "node": {
                          "exampleCompositions" : [ "ArcPoints.vuo" ]
                     }
                 });

void nodeEvent (
        VuoInputData(VuoInteger, {"default":36}) points,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0.5}) arcSize,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0.5}) xSize,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0.5}}) ySize,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0}) phase,
        VuoOutputData(VuoList_VuoPoint2d) positions
        )

{
    *positions = VuoListCreate_VuoPoint2d();

    VuoReal pointRad = (arcSize * (M_PI * 2)) / (points-1);
    VuoReal radPhase = (M_PI *2) * phase;

    for(int i=0; i<points; ++i) {

        VuoReal xVal = xSize * cos(radPhase + pointRad * i);
        VuoReal yVal = ySize * sin(radPhase + pointRad * i);

        VuoPoint2d position = VuoPoint2d_make(xVal,
                                              yVal);

        VuoListAppendValue_VuoPoint2d(*positions, position);
    }

}
