/**
 * @file
 * Grid implementation.
 *
 * @copyright Copyright © 2017–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                     "title" : "Make Cube Points",
                     "keywords" : [ ],
                     "version" : "1.0.0",
                     "dependencies" : [ ],
                     "node": {
                          "exampleCompositions" : [ "CubePoints.vuo" ]
                     }
                 });

void nodeEvent
(
        VuoInputData(VuoInteger, {"default": 8, "suggestedMin":1, "suggestedMax":1000}) pointsPerEdge,
        VuoInputData(VuoReal, {"default":1.0, "suggestedMin":0.0, "suggestedMax":2.0}) size,
        VuoInputData(VuoReal, {"default":0.0, "suggestedMin":-1.0, "suggestedMax":1.0}) offset,
        VuoInputData(VuoBoolean, {"default":false}) offsetFromSize,
        VuoInputData(VuoCurve,{"default":"linear"}) curve,
        VuoInputData(VuoCurveEasing, {"default": "in"}) easing,
        VuoOutputData(VuoList_VuoPoint3d) list
)
{
    //Create the output list
    *list = VuoListCreate_VuoPoint3d();

    //Create the point lists
    VuoList_VuoPoint3d startList = VuoListCreate_VuoPoint3d();
    VuoList_VuoPoint3d endList = VuoListCreate_VuoPoint3d();

    //Set amount of points per edge
    VuoInteger ppe = pointsPerEdge;

    //Calculate offset from size
    VuoReal calcOffset;

    if(offsetFromSize == true){
        calcOffset = size * offset;
    } else {
        calcOffset = offset;
    }

    //Get length from 0,0,0.
    VuoReal fc = size / 2;

    //Make vertex list
    //Make start points

        //Front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,-fc,fc + calcOffset)); //Btm left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,-fc,fc + calcOffset)); //Btm right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,fc,fc + calcOffset)); //Top right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,fc,fc + calcOffset)); //Top left front
        //Right
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc + calcOffset,-fc,fc)); //Btm right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc + calcOffset,-fc,-fc)); // Btm right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc + calcOffset,fc,-fc)); //Top right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc + calcOffset,fc,fc)); //Top right front
        //Back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,-fc,-fc - calcOffset)); //Btm left back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,-fc,-fc - calcOffset)); // Btm right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,fc,-fc - calcOffset)); //Top right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,fc,-fc - calcOffset)); //Top left back
        //Left
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc - calcOffset,-fc,fc)); //Btm left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc - calcOffset,-fc,-fc)); //Btm left back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc - calcOffset,fc,-fc)); //Top left back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc - calcOffset,fc,fc)); //Top left front
        //Top
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,fc + calcOffset,fc)); //Top left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,fc + calcOffset,fc)); //Top right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,fc + calcOffset,-fc)); //Top right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,fc + calcOffset,-fc)); //Top left back
        //Bottom
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,-fc - calcOffset,fc)); //Btm left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,-fc - calcOffset,fc)); //Btm right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fc,-fc - calcOffset,-fc)); // Btm right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fc,-fc - calcOffset,-fc)); //Btm left back

    //Make end points

        //Front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,-fc,fc + calcOffset)); //Btm right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,fc,fc + calcOffset)); //Top right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,fc,fc + calcOffset)); //Top left front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,-fc,fc + calcOffset)); //Btm left front
        //Right
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc + calcOffset,-fc,-fc)); // Btm right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc + calcOffset,fc,-fc)); //Top right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc + calcOffset,fc,fc)); //Top right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc + calcOffset,-fc,fc)); //Btm right front
        //Back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,-fc,-fc - calcOffset)); //Btm right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,fc,-fc - calcOffset)); //Top right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,fc,-fc - calcOffset)); //Top left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,-fc,-fc - calcOffset)); //Btm left Back
        //Left
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc - calcOffset,-fc,-fc)); //Btm left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc - calcOffset,fc,-fc)); //Top left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc - calcOffset,fc,fc)); //Top left front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc - calcOffset,-fc,fc)); //Btm left front
        //Top
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,fc + calcOffset,fc)); //Top right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,fc + calcOffset,-fc)); //Top right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,fc + calcOffset,-fc)); //Top left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,fc + calcOffset,fc)); //Top left front
        //Bottom
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,-fc - calcOffset,fc)); //Btm right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fc,-fc - calcOffset,-fc)); // Btm right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,-fc - calcOffset,-fc)); //Btm left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fc,-fc - calcOffset,fc)); //Btm left front

    for(int i = 0; i< VuoListGetCount_VuoPoint3d(startList); ++i)
        for(int j = 0; j < ppe; ++j){
            VuoPoint3d point = VuoPoint3d_curve(j,
                                                VuoListGetValue_VuoPoint3d(startList, i + 1),
                                                VuoListGetValue_VuoPoint3d(endList, i + 1),
                                                ppe-1,
                                                curve,
                                                easing,
                                                VuoLoopType_None);
            VuoListAppendValue_VuoPoint3d(*list, point);
        }

}
