/**
 * @file
 * Grid implementation.
 *
 * @copyright Copyright © 2017–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "math.h"

VuoModuleMetadata({
                     "title" : "Make Phyllotaxis Points",
                     "keywords" : [ ],
                     "version" : "1.0.0",
                     "dependencies" : [ ],
                     "node": {
                          "exampleCompositions" : [ "PhyllotaxisPoints.vuo" ]
                     }
                 });

/* Phyllotaxis pattern:
 * n = point number
 * c = scale
 * ø = n * 137.5
 * r = c * sqrt(n)
*/

void nodeEvent
(
        VuoInputData(VuoInteger, {"default": 12, "suggestedMin":12, "suggestedMax":2000}) points,
        VuoInputData(VuoReal, {"default":1.0, "suggestedMin":0.0, "suggestedMax":1.0}) size,
        VuoInputData(VuoReal, {"default":0.0, "suggestedMin":0.0, "suggestedMax":1.0}) offset,
        VuoInputData(VuoReal, {"default":137.5, "suggestedMin":0.0, "suggestedMax":360.0}) angle,
        VuoOutputData(VuoList_VuoPoint2d) list
)
{
    //Create the output list
    *list = VuoListCreate_VuoPoint2d();

    VuoReal setOffset = offset * (points - 1);
    VuoInteger calcOffset = floor(setOffset);
    VuoInteger offsetPoints = points + calcOffset;

    //loop to create the points; n = points, c = size
    for(unsigned int n = calcOffset; n < offsetPoints; ++n){
        VuoReal a = n * (angle * (M_PI / 180));
        VuoReal r = (size / sqrt(offsetPoints)) * sqrt(n);

        VuoPoint2d point = VuoPoint2d_make(r * cos(a), r * sin(a));

        VuoListAppendValue_VuoPoint2d(*list, point);
    }
}
