/**
 * @file
 * Grid implementation.
 *
 * @copyright Copyright © 2017–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                     "title" : "Make Rectangular Cuboid Points",
                     "keywords" : [ ],
                     "version" : "1.0.0",
                     "dependencies" : [ ],
                     "node": {
                          "exampleCompositions" : [ "RectangularCubePoints.vuo" ]
                     }
                 });

void nodeEvent
(
        VuoInputData(VuoInteger, {"default": 8, "suggestedMin":1, "suggestedMax":1000}) pointsPerEdge,
        VuoInputData(VuoPoint3d, {"default":{"x":1.0, "y":1.0, "z":1.0}, "suggestedMin":{"x":0.0, "y":0.0, "z":0.0}, "suggestedMax":{"x":2.0, "y":2.0, "z":2.0}}) size,
        VuoInputData(VuoPoint3d, {"default":{"x":0.0, "y":0.0, "z":0.0}, "suggestedMin":{"x":-1.0, "y":-1.0, "z":-1.0}, "suggestedMax":{"x":1.0, "y":1.0, "z":1.0}}) offset,
        VuoInputData(VuoBoolean, {"default":false}) offsetFromSize,
        VuoInputData(VuoCurve,{"default":"linear"}) curve,
        VuoInputData(VuoCurveEasing, {"default": "in"}) easing,
        VuoOutputData(VuoList_VuoPoint3d) list
)
{
    //Create the output list
    *list = VuoListCreate_VuoPoint3d();

    //Create the point lists
    VuoList_VuoPoint3d startList = VuoListCreate_VuoPoint3d();
    VuoList_VuoPoint3d endList = VuoListCreate_VuoPoint3d();

    //Set amount of points per edge
    VuoInteger ppe = pointsPerEdge;

    //Get length from 0,0,0.
    VuoReal fcx = size.x / 2;
    VuoReal fcy = size.y / 2;
    VuoReal fcz = size.z / 2;

    //Calculate offset from size
    VuoReal xOffset;
    VuoReal yOffset;
    VuoReal zOffset;

    if (offsetFromSize == true)
    {
        xOffset = size.x * offset.x;
        yOffset = size.y * offset.y;
        zOffset = size.z * offset.z;
    } else {
        xOffset = offset.x;
        yOffset = offset.y;
        zOffset = offset.z;
    }


    //Make vertex list
    //Make start points

        //Front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,-fcy,fcz + zOffset)); //Btm left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,-fcy,fcz + zOffset)); //Btm right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,fcy,fcz + zOffset)); //Top right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,fcy,fcz + zOffset)); //Top left front
        //Right
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx + xOffset,-fcy,fcz)); //Btm right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx + xOffset,-fcy,-fcz)); // Btm right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx + xOffset,fcy,-fcz)); //Top right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx + xOffset,fcy,fcz)); //Top right front
        //Back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,-fcy,-fcz - zOffset)); //Btm left back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,-fcy,-fcz - zOffset)); // Btm right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,fcy,-fcz - zOffset)); //Top right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,fcy,-fcz - zOffset)); //Top left back
        //Left
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx - xOffset,-fcy,fcz)); //Btm left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx - xOffset,-fcy,-fcz)); //Btm left back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx - xOffset,fcy,-fcz)); //Top left back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx - xOffset,fcy,fcz)); //Top left front
        //Top
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,fcy + yOffset,fcz)); //Top left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,fcy + yOffset,fcz)); //Top right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,fcy + yOffset,-fcz)); //Top right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,fcy + yOffset,-fcz)); //Top left back
        //Bottom
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,-fcy - yOffset,fcz)); //Btm left front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,-fcy - yOffset,fcz)); //Btm right front
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(fcx,-fcy - yOffset,-fcz)); // Btm right back
            VuoListAppendValue_VuoPoint3d(startList, VuoPoint3d_make(-fcx,-fcy - yOffset,-fcz)); //Btm left back

    //Make end points

        //Front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,-fcy,fcz + zOffset)); //Btm right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,fcy,fcz + zOffset)); //Top right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,fcy,fcz + zOffset)); //Top left front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,-fcy,fcz + zOffset)); //Btm left front
        //Right
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx + xOffset,-fcy,-fcz)); // Btm right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx + xOffset,fcy,-fcz)); //Top right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx + xOffset,fcy,fcz)); //Top right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx + xOffset,-fcy,fcz)); //Btm right front
        //Back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,-fcy,-fcz - zOffset)); //Btm right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,fcy,-fcz - zOffset)); //Top right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,fcy,-fcz - zOffset)); //Top left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,-fcy,-fcz - zOffset)); //Btm left Back
        //Left
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx - xOffset,-fcy,-fcz)); //Btm left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx - xOffset,fcy,-fcz)); //Top left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx - xOffset,fcy,fcz)); //Top left front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx - xOffset,-fcy,fcz)); //Btm left front
        //Top
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,fcy + yOffset,fcz)); //Top right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,fcy + yOffset,-fcz)); //Top right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,fcy + yOffset,-fcz)); //Top left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,fcy + yOffset,fcz)); //Top left front
        //Bottom
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,-fcy - yOffset,fcz)); //Btm right front
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(fcx,-fcy - yOffset,-fcz)); // Btm right back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,-fcy - yOffset,-fcz)); //Btm left back
            VuoListAppendValue_VuoPoint3d(endList, VuoPoint3d_make(-fcx,-fcy - yOffset,fcz)); //Btm left front

    for(int i = 0; i< VuoListGetCount_VuoPoint3d(startList); ++i)
        for(int j = 0; j < ppe; ++j){
            VuoPoint3d point = VuoPoint3d_curve(j,
                                                VuoListGetValue_VuoPoint3d(startList, i + 1),
                                                VuoListGetValue_VuoPoint3d(endList, i + 1),
                                                ppe-1,
                                                curve,
                                                easing,
                                                VuoLoopType_None);
            VuoListAppendValue_VuoPoint3d(*list, point);
        }

}
