/**
 * @file
 * Spherograph implementation.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "math.h"

VuoModuleMetadata({
                     "title" : "Make Spherograph Points",
                     "keywords" : ["Magneson", "Spirograph", "circles", "doodeling" ],
                     "version" : "1.1.0",
                     "dependencies" : [ ],
                     "node": {
                          "exampleCompositions" : [ "SpherographPoints.vuo" ]
                     }
                 });

void nodeEvent (
        VuoInputData(VuoInteger, {"default":100}) points,
        VuoInputData(VuoReal, {"default": 10.0}) thetaRounds,
        VuoInputData(VuoReal, {"default": 1.0}) phiRounds,
        VuoInputData(VuoList_VuoPoint3d) center,
        VuoInputData(VuoReal, {"default": 0.5}) radius,
        VuoOutputData(VuoList_VuoPoint3d) positions
        )

{
    // Create the output list

    *positions = VuoListCreate_VuoPoint3d();

    //Get maximum radians based on rounds

   VuoReal thetaRad = (M_PI * 2) * thetaRounds;
   VuoReal phiRad = (M_PI * 2) * phiRounds;

    //Convert amount of points to phi and theta base

    VuoReal theta = thetaRad / points;
    VuoReal phi = phiRad / points;

    //Calculate point values

    for(VuoInteger i = 0; i < points; ++i){

        VuoReal x =
                sin(theta * i)
                * sin(phi * i)
                * radius;

        VuoReal y =
                cos(theta * i)
                * sin(phi * i)
                * radius;

        VuoReal z =
                cos(phi * i)
                * radius;

        VuoReal xO = VuoListGetValue_VuoPoint3d(center, i+1).x;
        VuoReal yO = VuoListGetValue_VuoPoint3d(center, i+1).y;
        VuoReal zO = VuoListGetValue_VuoPoint3d(center, i+1).z;

        VuoPoint3d point = VuoPoint3d_make(x + xO, y + yO, z + zO);

        VuoListAppendValue_VuoPoint3d(*positions, point);
    }
}
