/**
 * @file
 * Spherograph implementation.
 *
 * @copyright Copyright © 2012–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "math.h"

VuoModuleMetadata({
                     "title" : "Make Spirograph Points",
                     "keywords" : ["Magneson", "Spirograph", "circles", "doodeling" ],
                     "version" : "1.1.0",
                     "dependencies" : [ ],
                     "node": {
                          "exampleCompositions" : [ "SpirographPoints.vuo" ]
                     }
                 });

void nodeEvent (
        VuoInputData(VuoInteger, {"default":1000}) points,
        VuoInputData(VuoReal, {"default":1}) rounds,
        VuoInputData(VuoList_VuoPoint3d) center,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0.5}) xSize,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":1.0}}) xSpeed,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0.}) xPhase,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0.5}}) ySize,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":1.0}) ySpeed,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0.25}) yPhase,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0}) zSize,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":1.0}}) zSpeed,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0}) zPhase,
        VuoInputData(VuoReal, {"suggestedMin":0, "suggestedMax":1, "default":0}) globalPhase,
        VuoOutputData(VuoList_VuoPoint3d) positions
        )

{
    //Set timebase per point
    VuoReal pointTime = rounds / (points -1);

    //Set timebase for sin/cos
    VuoReal piTime = 2 * M_PI;

    //Convert speed to Hz
    VuoReal xSpeedHz = piTime / xSpeed;
    VuoReal ySpeedHz = piTime / ySpeed;
    VuoReal zSpeedHz = piTime / zSpeed;

    //Set phase
    VuoReal xPhaseTime = ((1 / xSpeedHz) * (xPhase + globalPhase)) * (piTime);
    VuoReal yPhaseTime = ((1 / ySpeedHz) * (yPhase + globalPhase)) * (piTime);
    VuoReal zPhaseTime = ((1 / zSpeedHz) * (zPhase + globalPhase)) * (piTime);
    //Create the output list

    *positions = VuoListCreate_VuoPoint3d();

    for(int i=0; i<points; ++i) {

        VuoReal time = pointTime * i;

        //Calculate circles

        VuoReal circle1x = xSize * cos(xSpeedHz * (time + xPhaseTime));
        VuoReal circle1y = ySize * cos(ySpeedHz * (time + yPhaseTime));
        VuoReal circle1z = zSize * cos(zSpeedHz * (time + zPhaseTime));

        //Calculate points

        VuoReal x = VuoListGetValue_VuoPoint3d(center, i+1).x;
        VuoReal y = VuoListGetValue_VuoPoint3d(center, i+1).y;
        VuoReal z = VuoListGetValue_VuoPoint3d(center, i+1).z;

        VuoPoint3d position = VuoPoint3d_make(x + (circle1x), y + (circle1y), z + (circle1z));

        VuoListAppendValue_VuoPoint3d(*positions, position);
    }

}
